package com.sample;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderErrors;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.rule.FactHandle;

/**
 * This is a sample class to launch a rule.
 */
public class DroolsTest {

	public static final void main(String[] args) {
		try {
			// load up the knowledge base
			KnowledgeBase kbase = readKnowledgeBase();
			StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession();
			// go !
			
			Data data = new Data(1, 0);
			FactHandle dataHandle = ksession.insert(data);
			ksession.insert(new Data(2, 5));
			ksession.insert(new Data(3, 10));
			
			for(int i = 0; i < 10; i++) {
				ksession.insert(new Cycle(i));
				data.setValue(10-i);
				ksession.update(dataHandle, data);
				ksession.fireAllRules();
			}
			
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	private static KnowledgeBase readKnowledgeBase() throws Exception {
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		kbuilder.add(ResourceFactory.newClassPathResource("Sample.drl"), ResourceType.DRL);
		KnowledgeBuilderErrors errors = kbuilder.getErrors();
		if (errors.size() > 0) {
			for (KnowledgeBuilderError error: errors) {
				System.err.println(error);
			}
			throw new IllegalArgumentException("Could not parse knowledge.");
		}
		KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
		kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
		return kbase;
	}

	public static class Cycle {

		private final int index;

		public Cycle(int index) {
			this.index = index;
		}
		
		public int getIndex() {
			return this.index;
		}
		
		public String toString() {
			return "Cycle-" + index;
		}
	}

	public static class Data {

		private final int index;

		private int value;

		public Data(int index, int value) {
			this.index = index;
			this.value = value;
		}
		
		public int getValue() {
			return this.value;
		}
		
		public int getIndex() {
			return this.index;
		}
		
		public void setValue(int value) {
			this.value = value;
		}
		
		public String toString() {
			return "Data-" + index + " value " + value;
		}
	}

}